import json

from jsonpkt.commands import BaseCommand
from jsonpkt.utils import parse_packet_json, to_packet_json


class ViewCommand(BaseCommand):
    def process(self, args):
        file = args.file
        with open(file) as fp:
            packet_json = json.load(fp)
            packet = parse_packet_json(packet_json)
            if args.format == 'json':
                print(json.dumps(to_packet_json(packet), indent=args.indent))
            else:
                packet.show()



import os
import typing
from functools import reduce
from operator import truediv

from scapy import all as scapy


def replace_with_environment_var(value: typing.Union[int, str]) -> typing.Union[int, str]:
    if isinstance(value, str) and value.startswith('${') and value.endswith('}'):
        value = value[len('${'):-len('}')]
        value = os.getenv(value, '')
        
    return value


def resolve_environment_vars(packet_json: dict) -> dict:
    for key, value in packet_json.items():
        if key == 'payload':
            continue
        packet_json[key] = replace_with_environment_var(value)
        
    return packet_json


def parse_packet_json(packet_json: dict) -> scapy.Packet:
    packets = []
    while packet_type := next(iter(packet_json), None):
        packet = packet_json.get(packet_type, {})
        packet = resolve_environment_vars(packet)
        payload = packet.pop('payload', {})
        scapy_packet = getattr(scapy, packet_type, None)
        if scapy_packet:
            packets.append(scapy_packet(**packet))
        packet_json = payload
        
    return reduce(truediv, packets)


def to_packet_json(packet: scapy.Packet) -> dict:
    packet_jsons = []
    while not isinstance(packet, scapy.packet.NoPayload):
        packet_type = type(packet).__name__
        packet_fields = {
            key: value if isinstance(value, int) else str(value)
            for key, value in packet.fields.items()
            if key != 'options'
        }
        packet_jsons.append({ packet_type: packet_fields })
        packet = packet.payload
        
    for i in range(len(packet_jsons) - 1):
        if not packet_jsons[i]:
            continue
        packet_type, packet_fields = list(packet_jsons[i].items())[0]
        packet_fields['payload'] = packet_jsons[i + 1]
        
    return packet_jsons[0]



import argparse
import sys
from typing import List

from jsonpkt.commands import SendCommand, SendRecvCommand, ViewCommand


argparser = argparse.ArgumentParser()
subparsers = argparser.add_subparsers()

subparser_send = subparsers.add_parser('send', help='Generate and send packet by JSON file. See "jsonpkt send -h"')
subparser_send.add_argument('file', help='Packet definition file. (JSON)')
subparser_send.add_argument('-l', '--layer', required=False, default=3, type=int, help='Packet layer.')
subparser_send.set_defaults(handler=lambda args: SendCommand().process(args))

subparser_sendrecv = subparsers.add_parser('sendrecv', help='Generate, send and receive packet by JSON file. See "jsonpkt sendrecv -h"')
subparser_sendrecv.add_argument('file', help='Packet definition file. (JSON)')
subparser_sendrecv.add_argument('-l', '--layer', required=False, default=3, type=int, help='Packet layer.')
subparser_sendrecv.set_defaults(handler=lambda args: SendRecvCommand().process(args))

subparser_view = subparsers.add_parser('view', help='View packet prettierly. See "jsonpkt view -h"')
subparser_view.add_argument('file', help='Packet definition file. (JSON)')
subparser_view.add_argument('-f', '--format', required=False, default='scapy', help='Output format.')
subparser_view.add_argument('-i', '--indent', required=False, default=2, type=int, help='Indent width. (using format "json")')
subparser_view.set_defaults(handler=lambda args: ViewCommand().process(args))


def main(args: List[str] = sys.argv):
    _, *args = args
    parsed_args = argparser.parse_args(args=args)
    parsed_args.handler(parsed_args)
    



import json

from scapy import all as scapy

from jsonpkt.commands import BaseCommand
from jsonpkt.utils import parse_packet_json, to_packet_json


class SendRecvCommand(BaseCommand):
    def process(self, args):
        file = args.file
        with open(file) as fp:
            packet_json = json.load(fp)
            packet = parse_packet_json(packet_json)
            sender = scapy.srp1 if args.layer < 3 else scapy.sr1
            res = sender(packet, verbose=False)
            packet_json = to_packet_json(res)
            print(json.dumps(packet_json, indent=2))
